@@ -4,10 +4,10 @@ from toot import api, config
 from toot.auth import login_interactive, login_browser_interactive, create_app_interactive
 from toot.exceptions import ConsoleError, NotFoundError
 from toot.output import print_out, print_instance, print_account, print_search_results, print_timeline
-from toot.utils import assert_domain_exists
+from toot.utils import assert_domain_exists, multiline_input, EOF_KEY
 
 
-def timeline(app, user, args):
+def get_timeline_generator(app, user, args):
     # Make sure tag, list and public are not used simultaneously
     if len([arg for arg in [args.tag, args.list, args.public] if arg]) > 1:
         raise ConsoleError("Only one of --public, --tag, or --list can be used at one time.")
@@ -15,31 +15,63 @@ def timeline(app, user, args):
     if args.local and not (args.public or args.tag):
         raise ConsoleError("The --local option is only valid alongside --public or --tag.")
 
-    if args.public:
-        items = api.timeline_public(app, user, local=args.local)
-    elif args.tag:
-        items = api.timeline_tag(app, user, args.tag, local=args.local)
-    elif args.list:
-        items = api.timeline_list(app, user, args.list)
-    else:
-        items = api.timeline_home(app, user)
-
-    print_timeline(items)
-
-
-def curses(app, user, args):
-    from toot.ui.app import TimelineApp
-
-    if not args.public and (not app or not user):
-        raise ConsoleError("You must be logged in to view the home timeline.")
+    if args.instance and not (args.public or args.tag):
+        raise ConsoleError("The --instance option is only valid alongside --public or --tag.")
 
     if args.public:
         instance = args.instance or app.instance
-        generator = api.public_timeline_generator(instance)
+        return api.public_timeline_generator(instance, local=args.local, limit=args.count)
+    elif args.tag:
+        instance = args.instance or app.instance
+        return api.tag_timeline_generator(instance, args.tag, local=args.local, limit=args.count)
+    elif args.list:
+        return api.timeline_list_generator(app, user, args.list, limit=args.count)
     else:
-        generator = api.home_timeline_generator(app, user)
+        return api.home_timeline_generator(app, user, limit=args.count)
 
-    TimelineApp(generator).run()
+
+def timeline(app, user, args):
+    generator = get_timeline_generator(app, user, args)
+
+    while(True):
+        try:
+            items = next(generator)
+        except StopIteration:
+            print_out("That's all folks.")
+            return
+
+        if args.reverse:
+            items = reversed(items)
+
+        print_timeline(items)
+
+        if args.once:
+            break
+
+        char = input("\nContinue? [Y/n] ")
+        if char.lower() == "n":
+            break
+
+
+def thread(app, user, args):
+    toot = api.single_status(app, user, args.status_id)
+    context = api.context(app, user, args.status_id)
+    thread = []
+    for item in context['ancestors']:
+        thread.append(item)
+
+    thread.append(toot)
+
+    for item in context['descendants']:
+        thread.append(item)
+
+    print_timeline(thread)
+
+
+def curses(app, user, args):
+    generator = get_timeline_generator(app, user, args)
+    from toot.ui.app import TimelineApp
+    TimelineApp(app, user, generator).run()
 
 
 def post(app, user, args):
@@ -53,6 +85,10 @@ def post(app, user, args):
     if media and not args.text:
         args.text = media['text_url']
 
+    if not args.text:
+        print_out("Write or paste your toot. Press <yellow>{}</yellow> to post it.".format(EOF_KEY))
+        args.text = multiline_input()
+
     if not args.text:
         raise ConsoleError("You must specify either text or media to post.")
 
@@ -70,10 +106,44 @@ def post(app, user, args):
 
 def delete(app, user, args):
     api.delete_status(app, user, args.status_id)
-
     print_out("<green>✓ Status deleted</green>")
 
 
+def favourite(app, user, args):
+    api.favourite(app, user, args.status_id)
+    print_out("<green>✓ Status favourited</green>")
+
+
+def unfavourite(app, user, args):
+    api.unfavourite(app, user, args.status_id)
+    print_out("<green>✓ Status unfavourited</green>")
+
+
+def reblog(app, user, args):
+    api.reblog(app, user, args.status_id)
+    print_out("<green>✓ Status reblogged</green>")
+
+
+def unreblog(app, user, args):
+    api.unreblog(app, user, args.status_id)
+    print_out("<green>✓ Status unreblogged</green>")
+
+
+def pin(app, user, args):
+    api.pin(app, user, args.status_id)
+    print_out("<green>✓ Status pinned</green>")
+
+
+def unpin(app, user, args):
+    api.unpin(app, user, args.status_id)
+    print_out("<green>✓ Status unpinned</green>")
+
+
+def reblogged_by(app, user, args):
+    for account in api.reblogged_by(app, user, args.status_id):
+        print_out("{}\n @{}".format(account['display_name'], account['acct']))
+
+
 def auth(app, user, args):
     config_data = config.load_config()
 
@@ -93,7 +163,7 @@ def auth(app, user, args):
 
 
 def login_cli(app, user, args):
-    app = create_app_interactive(instance=args.instance)
+    app = create_app_interactive(instance=args.instance, scheme=args.scheme)
     login_interactive(app, args.email)
 
     print_out()
@@ -101,7 +171,7 @@ def login_cli(app, user, args):
 
 
 def login(app, user, args):
-    app = create_app_interactive(instance=args.instance)
+    app = create_app_interactive(instance=args.instance, scheme=args.scheme)
     login_browser_interactive(app)
 
     print_out()
@@ -216,7 +286,7 @@ def instance(app, user, args):
     assert_domain_exists(name)
 
     try:
-        instance = api.get_instance(name)
+        instance = api.get_instance(name, args.scheme)
         print_instance(instance)
     except NotFoundError:
         raise ConsoleError(
