@@ -4,7 +4,7 @@ import os
 import sys
 import logging
 
-from argparse import ArgumentParser, FileType
+from argparse import ArgumentParser, FileType, ArgumentTypeError
 from collections import namedtuple
 from toot import config, commands, CLIENT_NAME, CLIENT_WEBSITE, __version__
 from toot.exceptions import ApiError, ConsoleError
@@ -21,9 +21,17 @@ def visibility(value):
     return value
 
 
+def timeline_count(value):
+    n = int(value)
+    if not 0 < n <= 20:
+        raise ArgumentTypeError("Number of toots should be between 1 and 20.")
+    return n
+
+
 Command = namedtuple("Command", ["name", "description", "require_auth", "arguments"])
 
 
+# Aruguments added to every command
 common_args = [
     (["--no-color"], {
         "help": "don't use ANSI colors in output",
@@ -39,7 +47,14 @@ common_args = [
         "help": "show debug log in console",
         "action": 'store_true',
         "default": False,
-    })
+    }),
+]
+
+# Arguments added to commands which require authentication
+common_auth_args = [
+    (["-u", "--using"], {
+        "help": "the account to use, overrides active account",
+    }),
 ]
 
 account_arg = (["account"], {
@@ -56,18 +71,62 @@ email_arg = (["-e", "--email"], {
     "help": 'email address to log in with',
 })
 
+scheme_arg = (["--disable-https"], {
+    "help": "disable HTTPS and use insecure HTTP",
+    "dest": "scheme",
+    "default": "https",
+    "action": "store_const",
+    "const": "http",
+})
+
+status_id_arg = (["status_id"], {
+    "help": "ID of the status",
+    "type": int,
+})
+
+# Arguments for selecting a timeline (see `toot.commands.get_timeline_generator`)
+timeline_args = [
+    (["-p", "--public"], {
+        "action": "store_true",
+        "default": False,
+        "help": "show public timeline (does not require auth)",
+    }),
+    (["-t", "--tag"], {
+        "type": str,
+        "help": "show hashtag timeline (does not require auth)",
+    }),
+    (["-l", "--local"], {
+        "action": "store_true",
+        "default": False,
+        "help": "show only statuses from local instance (public and tag timelines only)",
+    }),
+    (["-i", "--instance"], {
+        "type": str,
+        "help": "mastodon instance from which to read (public and tag timelines only)",
+    }),
+    (["--list"], {
+        "type": int,
+        "help": "show timeline for given list.",
+    }),
+    (["-c", "--count"], {
+        "type": timeline_count,
+        "help": "number of toots to show per page (1-20, default 10).",
+        "default": 10,
+    }),
+]
+
 
 AUTH_COMMANDS = [
     Command(
         name="login",
         description="Log into a mastodon instance using your browser (recommended)",
-        arguments=[instance_arg],
+        arguments=[instance_arg, scheme_arg],
         require_auth=False,
     ),
     Command(
         name="login_cli",
         description="Log in from the console, does NOT support two factor authentication",
-        arguments=[instance_arg, email_arg],
+        arguments=[instance_arg, email_arg, scheme_arg],
         require_auth=False,
     ),
     Command(
@@ -90,6 +149,7 @@ AUTH_COMMANDS = [
     ),
 ]
 
+
 READ_COMMANDS = [
     Command(
         name="whoami",
@@ -115,7 +175,7 @@ READ_COMMANDS = [
                 "help": "instance domain (e.g. 'mastodon.social') or blank to use current",
                 "nargs": "?",
             }),
-
+            scheme_arg,
         ],
         require_auth=False,
     ),
@@ -134,27 +194,29 @@ READ_COMMANDS = [
         ],
         require_auth=True,
     ),
+    Command(
+        name="thread",
+        description="Show toot thread items",
+        arguments=[
+            (["status_id"], {
+                "help": "Show thread for toot.",
+            }),
+        ],
+        require_auth=True,
+    ),
     Command(
         name="timeline",
         description="Show recent items in a timeline (home by default)",
-        arguments=[
-            (["-p", "--public"], {
+        arguments=timeline_args + [
+            (["-r", "--reverse"], {
                 "action": "store_true",
                 "default": False,
-                "help": "Show public timeline.",
+                "help": "Reverse the order of the shown timeline (to new posts at the bottom)",
             }),
-            (["-t", "--tag"], {
-                "type": str,
-                "help": "Show timeline for given hashtag.",
-            }),
-            (["-i", "--list"], {
-                "type": int,
-                "help": "Show timeline for given list ID.",
-            }),
-            (["-l", "--local"], {
+            (["-1", "--once"], {
                 "action": "store_true",
                 "default": False,
-                "help": "Show only statuses from local instance (public and tag timelines only).",
+                "help": "Only show the first <count> toots, do not prompt to continue.",
             }),
         ],
         require_auth=True,
@@ -162,17 +224,7 @@ READ_COMMANDS = [
     Command(
         name="curses",
         description="An experimental timeline app (doesn't work on Windows)",
-        arguments=[
-            (["-p", "--public"], {
-                "action": 'store_true',
-                "default": False,
-                "help": "Resolve non-local accounts",
-            }),
-            (["-i", "--instance"], {
-                "type": str,
-                "help": 'instance from which to read (for public timeline only)',
-            })
-        ],
+        arguments=timeline_args,
         require_auth=False,
     ),
 ]
@@ -222,15 +274,55 @@ POST_COMMANDS = [
         ],
         require_auth=True,
     ),
+]
+
+STATUS_COMMANDS = [
     Command(
         name="delete",
-        description="Delete an existing status",
-        arguments=[
-            (["status_id"], {
-                "help": "ID of the status to delete",
-                "type": int,
-            })
-        ],
+        description="Delete a status",
+        arguments=[status_id_arg],
+        require_auth=True,
+    ),
+    Command(
+        name="favourite",
+        description="Favourite a status",
+        arguments=[status_id_arg],
+        require_auth=True,
+    ),
+    Command(
+        name="unfavourite",
+        description="Unfavourite a status",
+        arguments=[status_id_arg],
+        require_auth=True,
+    ),
+    Command(
+        name="reblog",
+        description="Reblog a status",
+        arguments=[status_id_arg],
+        require_auth=True,
+    ),
+    Command(
+        name="unreblog",
+        description="Unreblog a status",
+        arguments=[status_id_arg],
+        require_auth=True,
+    ),
+    Command(
+        name="reblogged_by",
+        description="Show accounts that reblogged the status",
+        arguments=[status_id_arg],
+        require_auth=False,
+    ),
+    Command(
+        name="pin",
+        description="Pin a status",
+        arguments=[status_id_arg],
+        require_auth=True,
+    ),
+    Command(
+        name="unpin",
+        description="Unpin a status",
+        arguments=[status_id_arg],
         require_auth=True,
     ),
 ]
@@ -286,7 +378,7 @@ ACCOUNTS_COMMANDS = [
     ),
 ]
 
-COMMANDS = AUTH_COMMANDS + READ_COMMANDS + POST_COMMANDS + ACCOUNTS_COMMANDS
+COMMANDS = AUTH_COMMANDS + READ_COMMANDS + POST_COMMANDS + STATUS_COMMANDS + ACCOUNTS_COMMANDS
 
 
 def print_usage():
@@ -296,6 +388,7 @@ def print_usage():
         ("Authentication", AUTH_COMMANDS),
         ("Read", READ_COMMANDS),
         ("Post", POST_COMMANDS),
+        ("Status", STATUS_COMMANDS),
         ("Accounts", ACCOUNTS_COMMANDS),
     ]
 
@@ -323,12 +416,12 @@ def get_argument_parser(name, command):
         description=command.description,
         epilog=CLIENT_WEBSITE)
 
-    for args, kwargs in command.arguments + common_args:
-        parser.add_argument(*args, **kwargs)
-
-    # If the command requires auth, give an option to select account
+    combined_args = command.arguments + common_args
     if command.require_auth:
-        parser.add_argument("-u", "--using", help="the account to use, overrides active account")
+        combined_args += common_auth_args
+
+    for args, kwargs in combined_args:
+        parser.add_argument(*args, **kwargs)
 
     return parser
 
