@@ -3,13 +3,13 @@
 import sys
 import re
 
-from bs4 import BeautifulSoup
 from datetime import datetime
-from itertools import chain
-from itertools import zip_longest
-from textwrap import wrap, TextWrapper
+from textwrap import wrap
+from wcwidth import wcswidth
+
+from toot.utils import format_content, get_text, parse_html
+from toot.wcstring import wc_wrap
 
-from toot.utils import format_content, get_text, trunc
 
 START_CODES = {
     'red': '\033[31m',
@@ -99,6 +99,13 @@ def print_account(account):
     print_out(account['url'])
 
 
+HASHTAG_PATTERN = re.compile(r'(?<!\w)(#\w+)\b')
+
+
+def highlight_hashtags(line):
+    return re.sub(HASHTAG_PATTERN, '<cyan>\\1</cyan>', line)
+
+
 def print_search_results(results):
     accounts = results['accounts']
     hashtags = results['hashtags']
@@ -119,47 +126,52 @@ def print_search_results(results):
         print_out("<yellow>Nothing found</yellow>")
 
 
-def print_timeline(items):
-    def _print_item(item):
-        def wrap_text(text, width):
-            wrapper = TextWrapper(width=width, break_long_words=False, break_on_hyphens=False)
-            return chain(*[wrapper.wrap(l) for l in text.split("\n")])
+def print_status(status, width):
+    reblog = status['reblog']
+    content = reblog['content'] if reblog else status['content']
+    media_attachments = reblog['media_attachments'] if reblog else status['media_attachments']
+    in_reply_to = status['in_reply_to_id']
 
-        def timeline_rows(item):
-            display_name = item['account']['display_name']
-            username = "@" + item['account']['username']
-            time = item['time'].strftime('%Y-%m-%d %H:%M%Z')
+    time = status['created_at']
+    time = datetime.strptime(time, "%Y-%m-%dT%H:%M:%S.%fZ")
+    time = time.strftime('%Y-%m-%d %H:%M%z')
 
-            left_column = [display_name]
-            if display_name != username:
-                left_column.append(username)
-            left_column.append(time)
-            if item['reblogged']:
-                left_column.append("Reblogged @{}".format(item['reblogged']))
+    username = "@" + status['account']['acct']
+    spacing = width - wcswidth(username) - wcswidth(time)
 
-            right_column = wrap_text(item['text'], 80)
+    display_name = status['account']['display_name']
+    if display_name:
+        spacing -= wcswidth(display_name) + 1
 
-            return zip_longest(left_column, right_column, fillvalue="")
+    print_out("{}{}{}{}".format(
+        "<green>{}</green> ".format(display_name) if display_name else "",
+        "<blue>{}</blue>".format(username),
+        " " * spacing,
+        "<yellow>{}</yellow>".format(time),
+    ))
 
-        for left, right in timeline_rows(item):
-            print_out("{:30} │ {}".format(trunc(left, 30), right))
+    for paragraph in parse_html(content):
+        print_out("")
+        for line in paragraph:
+            for subline in wc_wrap(line, width):
+                print_out(highlight_hashtags(subline))
 
-    def _parse_item(item):
-        content = item['reblog']['content'] if item['reblog'] else item['content']
-        reblogged = item['reblog']['account']['username'] if item['reblog'] else None
+    if media_attachments:
+        print_out("\nMedia:")
+        for attachment in media_attachments:
+            url = attachment['text_url'] or attachment['url']
+            for line in wc_wrap(url, width):
+                print_out(line)
 
-        soup = BeautifulSoup(content, "html.parser")
-        text = soup.get_text().replace('&apos;', "'")
-        time = datetime.strptime(item['created_at'], "%Y-%m-%dT%H:%M:%S.%fZ")
+    print_out("\n{}{}{}".format(
+        "ID <yellow>{}</yellow>  ".format(status['id']),
+        "↲ In reply to <yellow>{}</yellow>  ".format(in_reply_to) if in_reply_to else "",
+        "↻ Reblogged <blue>@{}</blue>  ".format(reblog['account']['acct']) if reblog else "",
+    ))
 
-        return {
-            "account": item['account'],
-            "text": text,
-            "time": time,
-            "reblogged": reblogged,
-        }
 
-    print_out("─" * 31 + "┬" + "─" * 88)
+def print_timeline(items, width=100):
+    print_out("─" * width)
     for item in items:
-        _print_item(_parse_item(item))
-        print_out("─" * 31 + "┼" + "─" * 88)
+        print_status(item, width)
+        print_out("─" * width)
