@@ -15,8 +15,15 @@ def parse_status(status):
         'boosted_by': boosted_by,
         'created_at': created_at,
         'content': content,
+        'favourited': status.get('favourited'),
+        'favourites_count': _status['favourites_count'],
+        'id': status['id'],
+        'in_reply_to_id': _status.get('in_reply_to_id'),
         'media_attachments': _status['media_attachments'],
         'url': _status['url'],
+        'reblogged': status.get('reblogged'),
+        'reblogs_count': _status['reblogs_count'],
+        'replies_count': _status.get('replies_count', 0),
         'spoiler_text': spoiler_text,
         'sensitive': _status['sensitive'],
         'show_sensitive': False,
